#G Modulinhalt: Lineare Algebra
#E content: linear algebra
# 
# written by: Mario Chemnitz (ucla@hrz.tu-chemnitz.de)
# revision: $Id: matrix.py,v 1.11 1996/10/20 22:20:10 che Exp $
#
#E introduces an English comment
#G leitet einen deutschen Kommentar ein

print "I'm loading matrix.py ...",

from math import sqrt,acos,sin,cos
from string import joinfields

#G allgemeine Funktionen
#E some common functions
def isMatrix(obj):
  return hasattr(obj,'isMatrix') and obj.isMatrix

def isVector(obj):
  return hasattr(obj,'isVector') and obj.isVector
  
#G Matrixklasse
#E matrix class
class Matrix:
  isMatrix=1
 
  def __init__(self,m=3,n=3,init=[]):
    #G m,n - Zeilen,Spalten
    #E m,n - rows,columns
    self.m=m
    self.n=n
    #G Datenliste
    #E data list
    self.data=[]
    if init:
      if (type(init) == type(self.data)) and (len(init)==len(self)):
        #G Daten stammen aus einer Liste
	#E data come from a list
        self.data=init
      else:
        raise TypeError, "init is wrong"
    else:
      for element in range(self.m*self.n):
        self.data.append(0.0)
 
  #G Darstellung durch 'print' usw.
  #E representation
  def __repr__(self):
    Liste=[' ']
    for i in range(self.m):
      for j in range(self.n):
        Liste.append(str(round(self[i:j],4)))
      Liste.append('\n')
    return joinfields(Liste,'\t')    
 
  #G Matrix mit Daten f"ullen
  #E fills the matrix with data
  def set(self,init):
    if (type(init) == type(self.data)) and (len(init)==len(self)):
      #G Daten stammen aus einer Liste
      #E data come from a list
      self.data=init
    elif (type(init) == type(self)) and (init.m==self.m) and (init.n==self.n):
      #G Daten stammen von einer anderen Matrixinstanz
      #E data come from another matrix instance
      self.data=init.data
    else:
      raise TypeError,"m-n-assign error"
  
  #G Addition: -> Matrix
  #E addition: -> matrix
  def __add__(self,other):
    if isMatrix(other) and (self.m==other.m) and (self.n==other.n):
      tmp=Matrix(self.m,self.n)
      for i in range(len(self)):
        tmp[i]=self[i]+other[i]
      return tmp
    else:
      raise TypeError,"error in matrix addition"

  #G rechtsseitige Addition: -> Matrix
  #E operand for addition is on the right: -> matrix
  __radd__=__add__
  
  #G Subtraktion: -> Matrix
  #E subtraction: -> matrix
  def __sub__(self,other):
    if isMatrix(other) and (self.m==other.m) and (self.n==other.n):
      tmp=Matrix(self.m,self.n)
      for i in range(len(self)):
        tmp[i]=self[i]-other[i]
      return tmp
    else:
      raise TypeError,"error in matrix subtraction"
  
  #G rechtsseiteige Subtraktion: -> Matrix
  #E operand for subtraction is on the right: -> matrix
  def __rsub__(self,other):
    if isMatrix(other) and (self.m==other.m) and (self.n==other.n):
      tmp=Matrix(self.m,self.n)
      for i in range(len(self)):
        tmp[i]=other[i]-self[i]
      return tmp
    else:
      raise TypeError,"error in  right matrix subtraction"

  #G Negation der Elemente: -> Matrix
  #E negation of the entries: -> matrix
  def __neg__(self):
    tmp=Matrix(self.m,self.n)
    for i in range(len(self)):
      tmp[i]=-self[i]
    return tmp
      
  #G Elementzahl: -> int
  #E number of entries: -> integer value
  def __len__(self):
    return self.m*self.n

  #G Element Nummer i aus Datenliste lesen: -> Element
  #E reads entry number i from data list: -> data elememt
  def __getitem__(self,i):
    if i < (len(self)):
      return self.data[i]
    else:
      raise IndexError,"index reading error"
    
  #G Element in Datenliste schreiben
  #E writes element to data list
  def __setitem__(self,i,other):
    if i < (len(self)):
      self.data[i]=other
    else:
      raise IndexError,"index writing error"
    
  #G Element per i,j-Index aus Matrix lesen: -> Element
  #E reads entry from row i and column j: -> data element
  def __getslice__(self,i,j):
    if i<self.m and j<self.n:
      return self.data[self.n*i+j]
    else:
      raise IndexError,"index reading error"

  #G Element per i,j-Index in Matrix schreiben
  #E writes matrix element to row i and column j
  def __setslice__(self,i,j,wert):
    if i<self.m and j<self.n:
      self.data[self.n*i+j]=wert
    else:
      raise IndexError,"index writing error"

  #G Matrizenmultiplikation (self*other): -> Matrix oder Vektor
  #E matrix multiplication (self*other): -> matrix or vector
  def __mul__(self,other):
    if isMatrix(other) and (self.n==other.m):
      tmp=Matrix(self.m,other.n)
      for i in range(tmp.m):
        for j in range(tmp.n):
          for k in range(self.n):
            tmp[i:j]=tmp[i:j]+self[i:k]*other[k:j]
      return tmp
    elif isVector(other) and (self.m==3) and (self.n==3):
      tmp=Vector()
      for i in range(tmp.m):
        for k in range(self.n):
          tmp[i]=tmp[i]+self[i:k]*other[k]
      return tmp      
    elif (type(other)==type(1.0)) or (type(other)==type(1)):
      tmp=Matrix(self.m,self.n)
      for i in range(len(self)):
        tmp[i]=self[i]*other
      return tmp	
    else:
      raise TypeError,"m-n-error in matrix multiplication"
      
  #G rechtsseitige Matrizenmultiplikation (other*self): -> Matrix
  #E operand for matrix multiplication is on the right (other*self): -> matrix
  def __rmul__(self,other):
    if (type(self)!=type(other)):
      tmp=Matrix(self.m,self.n)
      for i in range(len(self)):
        tmp[i]=other*self[i]
      return tmp	
    else:
      raise TypeError,"error in right matrix multiplication"

  #G Determinante einer (3,3)-Matrix: -> Skalar
  #E determinant of a (3,3)-matrix: -> scalar value
  def det(self):
    if(self.m == 3 and self.n == 3):
      return (self[0:0]*self[1:1]*self[2:2] - self[0:0]*self[2:1]*self[1:2] \
            - self[0:1]*self[1:0]*self[2:2] + self[0:1]*self[2:0]*self[1:2] \
	    + self[0:2]*self[1:0]*self[2:1] - self[0:2]*self[2:0]*self[1:1])
    else:
      raise TypeError, "determinants of third order defined only"
  
#G vier spezielle (3,3)-Matrixinstanzen
#E four special (3,3)-matrix instances

#G Transformationsmatrix f"ur Rotation um x-Achse
#E matrix for rotation on the x-axis
def Rx(w=0):
  return Matrix(3,3,
      [1.0, 0.0,     0.0,
       0.0, cos(w), -sin(w),
       0.0, sin(w),  cos(w)])

#G Transformationsmatrix f"ur Rotation um y-Achse
#E matrix for rotation on the y-axis
def Ry(w=0):
  return Matrix(3,3,
      [cos(w), 0.0, sin(w),
       0.0,    1.0, 0.0,
      -sin(w), 0.0, cos(w)])

#G Transformationsmatrix f"ur Rotation um z-Achse
#E matrix for rotation on the z-axis
def Rz(w=0):
  return Matrix(3,3,
      [cos(w), -sin(w), 0.0,
       sin(w),  cos(w), 0.0,
       0.0,     0.0,    1.0])
  
#G Einheitsmatrix
#E identity matrix
E=Matrix(3,3,
      [1.0, 0.0, 0.0,
       0.0, 1.0, 0.0,
       0.0, 0.0, 1.0])

#G Vektorklasse, (3,1)-Vektoren
#E vector class, (3,1)-vectors
class Vector(Matrix):
  isVector=1
  
  isMatrix=0

  def __init__(self,x=0.0,y=0.0,z=0.0):
    Matrix.__init__(self,3,1,[x,y,z])

  #G Addition: -> Vektor
  #E addition: -> vector
  def __add__(s,o):
    if isVector(o):
      return Vector(s[0]+o[0],s[1]+o[1],s[2]+o[2])
    else:
      raise TypeError, "other is no vector in vector addition"

  #G rechtsseitige Addition: -> Vektor
  #E operand for addition is on the right: -> vector
  __radd__=__add__

  #G Subtraktion: -> Vektor
  #E subtraction: -> vector
  def __sub__(s,o):
    if isVector(o):
      return Vector(s[0]-o[0],s[1]-o[1],s[2]-o[2])
    else:
      raise TypeError, "other is no vector in vector subtraction"

  #G rechtsseitige Subtraktion: -> Vektor
  #E operand for subtraction is on the right: -> vector
  def __rsub__(s,o):
    if isVector(o):
      return Vector(o[0]-s[0],o[1]-s[1],o[2]-s[2])
    else:
      raise TypeError, "other is no vector in right vector subtraction"

  #G Negation: -> Vektor
  #E negation: -> vector
  def __neg__(self):
    return Vector(-self[0],-self[1],-self[2])


  #G Skalarplodukt oder Produkt mit einem Wert: -> Vektor oder Skalar
  #E scalar product or multiplication by a scalar value: -> scalar value or vector
  def __mul__(self,other):
    if isVector(other):
      summe=0.0
      for i in range(3):
        summe=summe+self[i]*other[i]
      return summe
    elif isMatrix(other):
      raise TypeError, "other must not be a matrix"
    else:
      return Vector(self[0]*other,self[1]*other,self[2]*other)

  #G rechtsseitiges Skalarprodukt: -> Vektor
  #E operand for scalar product is on the right: -> vector
  def __rmul__(self,other):
    return Vector(self[0]*other,self[1]*other,self[2]*other)
  
  #G Kreuzprodukt zweier Vektoren: -> Vektor
  #E cross product: -> vector
  def cross(self,other):
    if isVector(other):
      tmp=Vector()
      tmp[0]=self[1]*other[2]-self[2]*other[1]
      tmp[1]=self[2]*other[0]-self[0]*other[2]
      tmp[2]=self[0]*other[1]-self[1]*other[0]
      return tmp
    else:
      raise TypeError, "other is no vector in cross product"
  
  #G L"ange des Vektors self: -> Skalar
  #E length of a vector: -> scalar value
  def betrag(self):
    return sqrt(self*self)
  
  length=betrag
  
  #G Normalenvektor in Richtung von self: -> Vektor
  #E vector of length 1 in direction of self: -> vector
  def normal(self):
    betr=self.betrag()
    if betr == 0:
      raise ZeroDivisionError, "self is a zero-length vector"
    else:
      return (1/betr)*self
  
  #G Winkel zwischen zwei Vektoren (self, other): -> Winkel in rad
  #E angle (rad) between two vectors (self, other): -> angle
  def winkel(self,other):
    if isVector(other):
      tmp=1/(self.betrag()*other.betrag())*(self*other)
      if tmp>=-1.0 and tmp <=1.0:
        return acos(tmp)
      else:
        raise "domain error in function acos()!"
    else:
      raise TypeError, "other is no vector"
      
  angle=winkel

#G Einheitsvektoren
#E base vectors
ex=Vector(1.,0.,0.)
ey=Vector(0.,1.,0.)
ez=Vector(0.,0.,1.)

print "ok!"

#G Objekte zu Testzwecken
#E objects for tests
A=Matrix()
A.set([1.0,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0]) 
B=Matrix()
B.set([1.1,2.2,3.3,4.4,5.5,6.6,7.7,8.8,9.9])
C=Matrix(3,1,[4,4,4])
D=Matrix(3,3,[-23, 1.9, 14.0, 0, 123.45, -99.1, 0.0003, 0.1, 1.0])

v=Vector( 1.1, 2.2, 3.3)
w=Vector(11.1,22.2,33.3)

